package form;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;

// |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
// |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

/**
 * This class is a customizer for the Producer bean.  It displays a
 * TextArea and three TextFields where the user can enter the dialog message
 * and the labels for each of the three buttons.  It does not allow the
 * dialog title or other resources to be set.
 */

public class FormCustomizer extends Panel
       implements Customizer, ActionListener
{
  // |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
  // |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
  // ======= Variable Declarations =========

  protected Form bean;                 // The bean being customized
  protected TextComponent title   ;        // Title of the bean 
  protected TextComponent description  ;   // Description of the bean
  protected TextComponent parametersURL ;  // Description of the bean
  protected Button doneButton ;            // Customization done button


  // |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
  // |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

  // The bean box calls this method to tell us what object to customize.
  // This method will always be called before the customizer is displayed,
  // so it is safe to create the customizer GUI here.

  public void setObject ( Object o )  {

     bean = ( Form ) o ;   // save the object we're customizing

     // Put a label at the top of the panel.
     GridBagLayout mainLayout = new GridBagLayout( ) ;
     this.setLayout( mainLayout );
     GridBagConstraints mc = new GridBagConstraints();
     mc.weightx = 1.0;
     mc.gridwidth =  GridBagConstraints.REMAINDER ;
     mc.anchor =  GridBagConstraints.WEST ;
     mc.insets = new Insets ( 10,  10, 10, 10 ) ;

     // ==== Add the instruction panel ======

     Panel instPanel = new Panel ( ) ;
     instPanel.setLayout ( new GridLayout ( 1,1,0,0 )) ;
     instPanel.setFont(new Font("Helvetica", Font.BOLD, 12));
     Label instLabel = new Label ("Enter the appropriate details and " +
				  "click button" ) ;
     instPanel.add ( instLabel ) ;
     mainLayout.setConstraints ( instPanel, mc ) ;
     this.add ( instPanel ) ;

     // ==== Add the bean title panel ========

     mc.anchor =  GridBagConstraints.WEST ;
     Panel titlePanel = new Panel() ;
     titlePanel.setFont(new Font("Helvetica", Font.BOLD, 12));
     GridBagLayout titleLayout = new GridBagLayout () ;
     GridBagConstraints c = new GridBagConstraints();
     c.weightx = 1.0;
     c.gridwidth =  GridBagConstraints.REMAINDER ;
     c.anchor =  GridBagConstraints.WEST ;

     titlePanel.setLayout (titleLayout) ;
     mainLayout.setConstraints (titlePanel, mc) ;
     this.add (titlePanel) ;

     titlePanel.setFont(new Font("Helvetica", Font.PLAIN, 12));
     c.weightx = 0.3 ;
     c.fill = GridBagConstraints.HORIZONTAL ;
     c.gridwidth =  GridBagConstraints.RELATIVE ;
     Label titleLabel = new Label ("Bean Title : ") ;
     titleLayout.setConstraints (titleLabel, c) ;     
     titlePanel.add (titleLabel) ;
     c.gridwidth = GridBagConstraints.REMAINDER ;
     title = (TextComponent) new TextField (bean.getTitle(), 40) ;
     titleLayout.setConstraints (title, c) ;     
     titlePanel.add (title) ;

     // ====== Bean Description =====     

     c.gridwidth = 1 ;
     Label descLabel = new Label("Description : ") ;
     titleLayout.setConstraints (descLabel, c) ;
     titlePanel.add (descLabel) ;
     c.gridwidth = 1 ;
     description = new TextArea (bean.getDescription(), 4, 34) ;
     c.fill = GridBagConstraints.HORIZONTAL ;
     c.gridwidth = GridBagConstraints.REMAINDER ;
     titleLayout.setConstraints (description, c) ;     
     titlePanel.add (description) ;

     // ====== Parameter Set URL =====     

     c.gridwidth = 1 ;
     Label parmLabel = new Label("Parameter Set URL : ") ;
     titleLayout.setConstraints (parmLabel, c) ;
     titlePanel.add (parmLabel) ;
     c.gridwidth = 1 ;
     parametersURL = (TextComponent) new TextField (bean.getParameterSet(),40);
     c.fill = GridBagConstraints.HORIZONTAL ;
     c.gridwidth = GridBagConstraints.REMAINDER ;
     titleLayout.setConstraints (parametersURL, c) ;     
     titlePanel.add (parametersURL) ;

     // ===== Add the Done button panel =======

     mc.anchor = GridBagConstraints.CENTER ;
     Panel buttonbox = new Panel();
     buttonbox.setLayout( new GridLayout (1,1,0,0)) ; 
     doneButton = new Button ( "Apply Customization" ) ;
     doneButton.setFont(new Font("Helvetica", Font.PLAIN, 12));
     doneButton.addActionListener ( this ) ;
     doneButton.setSize ( 250, 30 ) ;
     buttonbox.add(doneButton);
     mainLayout.setConstraints ( buttonbox, mc ) ;     
     this.add(buttonbox);
  }

  // |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
  // |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
  // Add some space around the outside of the panel.

  public Insets getInsets() {
 
    return new Insets(10, 10, 10, 10) ; 
  }
 
  // |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
  // |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

  // This is the method defined by the ActionListener interface.  Whenever the
  // user clicks the done button, this will get called. It updates the state
  // of the bean being customized and makes it ready for deployment. 

  public void actionPerformed (ActionEvent e) {
    
    if ( e.getSource( ) == doneButton ) {
      bean.setTitle (title.getText()) ;
      bean.setDescription (description.getText()) ;
      bean.setParameterSet (parametersURL.getText()) ;
      listeners.firePropertyChange ( null, null, null ) ;
    }
  }

  // |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
  // |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||

  // This code uses the PropertyChangeSupport class to maintain a list of
  // listeners interested in the edits we make to the bean.
  protected PropertyChangeSupport listeners = new PropertyChangeSupport(this);

  public void addPropertyChangeListener(PropertyChangeListener l) {
    listeners.addPropertyChangeListener(l);
  }

  public void removePropertyChangeListener(PropertyChangeListener l) {
    listeners.removePropertyChangeListener(l);
  }
}

// |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
// |||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||



